@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2020 by the Free Software Foundation, Inc.
+# Copyright (C) 2006-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -22,7 +22,8 @@ import os
 from mailman.config import config
 from mailman.database.model import Model
 from mailman.database.transaction import dbconnection
-from mailman.database.types import Enum, SAUnicode, SAUnicodeLarge
+from mailman.database.types import (
+    Enum, SAText, SAUnicode, SAUnicode4Byte, SAUnicodeLarge)
 from mailman.interfaces.action import Action, FilterAction
 from mailman.interfaces.address import IAddress, InvalidEmailAddressError
 from mailman.interfaces.archiver import ArchivePolicy
@@ -33,9 +34,10 @@ from mailman.interfaces.digests import DigestFrequency
 from mailman.interfaces.domain import IDomainManager
 from mailman.interfaces.languages import ILanguageManager
 from mailman.interfaces.mailinglist import (
-    DMARCMitigateAction, IAcceptableAlias, IAcceptableAliasSet,
-    IHeaderMatch, IHeaderMatchList, IListArchiver, IListArchiverSet,
-    IMailingList, Personalization, ReplyToMunging, SubscriptionPolicy)
+    ArchiveRenderingMode, DMARCMitigateAction, IAcceptableAlias,
+    IAcceptableAliasSet, IHeaderMatch, IHeaderMatchList, IListArchiver,
+    IListArchiverSet, IMailingList, Personalization, ReplyToMunging,
+    SubscriptionPolicy)
 from mailman.interfaces.member import (
     AlreadySubscribedError, MemberRole, MembershipIsBannedError,
     MissingPreferredAddressError, SubscriptionEvent)
@@ -108,11 +110,11 @@ class MailingList(Model):
     # Automatic responses.
     autoresponse_grace_period = Column(Interval)
     autorespond_owner = Column(Enum(ResponseAction))
-    autoresponse_owner_text = Column(SAUnicode)
+    autoresponse_owner_text = Column(SAText)
     autorespond_postings = Column(Enum(ResponseAction))
-    autoresponse_postings_text = Column(SAUnicode)
+    autoresponse_postings_text = Column(SAText)
     autorespond_requests = Column(Enum(ResponseAction))
-    autoresponse_request_text = Column(SAUnicode)
+    autoresponse_request_text = Column(SAText)
     # Content filters.
     filter_action = Column(Enum(FilterAction))
     filter_content = Column(Boolean)
@@ -120,7 +122,8 @@ class MailingList(Model):
     convert_html_to_plaintext = Column(Boolean)
     # Bounces.
     bounce_info_stale_after = Column(Interval)
-    bounce_matching_headers = Column(SAUnicode)                  # XXX
+    bounce_matching_headers = Column(SAUnicode4Byte)              # XXX
+    bounce_notify_owner_on_bounce_increment = Column(Boolean)
     bounce_notify_owner_on_disable = Column(Boolean)
     bounce_notify_owner_on_removal = Column(Boolean)
     bounce_score_threshold = Column(Integer)
@@ -137,7 +140,7 @@ class MailingList(Model):
     # Miscellaneous
     default_member_action = Column(Enum(Action))
     default_nonmember_action = Column(Enum(Action))
-    description = Column(SAUnicode)
+    description = Column(SAUnicode4Byte)
     digests_enabled = Column(Boolean)
     digest_is_default = Column(Boolean)
     digest_send_periodic = Column(Boolean)
@@ -151,17 +154,17 @@ class MailingList(Model):
     gateway_to_mail = Column(Boolean)
     gateway_to_news = Column(Boolean)
     hold_these_nonmembers = Column(MutableList.as_mutable(PickleType))
-    info = Column(SAUnicode)
+    info = Column(SAText)
     linked_newsgroup = Column(SAUnicode)
     max_days_to_hold = Column(Integer)
     max_message_size = Column(Integer)
     max_num_recipients = Column(Integer)
-    member_moderation_notice = Column(SAUnicode)
+    member_moderation_notice = Column(SAUnicode4Byte)
     # FIXME: There should be no moderator_password
     moderator_password = Column(LargeBinary)             # TODO : was RawStr()
     newsgroup_moderation = Column(Enum(NewsgroupModeration))
     nntp_prefix_subject_too = Column(Boolean)
-    nonmember_rejection_notice = Column(SAUnicode)
+    nonmember_rejection_notice = Column(SAUnicode4Byte)
     obscure_addresses = Column(Boolean)
     owner_chain = Column(SAUnicode)
     owner_pipeline = Column(SAUnicode)
@@ -170,7 +173,7 @@ class MailingList(Model):
     posting_chain = Column(SAUnicode)
     posting_pipeline = Column(SAUnicode)
     _preferred_language = Column('preferred_language', SAUnicode)
-    display_name = Column(SAUnicode)
+    display_name = Column(SAUnicode4Byte)
     reject_these_nonmembers = Column(MutableList.as_mutable(PickleType))
     reply_goes_to_list = Column(Enum(ReplyToMunging))
     reply_to_address = Column(SAUnicode)
@@ -180,13 +183,14 @@ class MailingList(Model):
     scrub_nondigest = Column(Boolean)
     send_goodbye_message = Column(Boolean)
     send_welcome_message = Column(Boolean)
-    subject_prefix = Column(SAUnicode)
+    subject_prefix = Column(SAUnicode4Byte)
     subscription_policy = Column(Enum(SubscriptionPolicy))
     topics = Column(PickleType)
     topics_bodylines_limit = Column(Integer)
     topics_enabled = Column(Boolean)
     unsubscription_policy = Column(Enum(SubscriptionPolicy))
     usenet_watermark = Column(Integer)
+    archive_rendering_mode = Column(Enum(ArchiveRenderingMode))
     # ORM relationships.
     header_matches = relationship(
         'HeaderMatch', backref='mailing_list',
@@ -651,7 +655,7 @@ class HeaderMatch(Model):
 
     _position = Column('position', Integer, index=True, default=0)
     header = Column(SAUnicode)
-    pattern = Column(SAUnicode)
+    pattern = Column(SAUnicode4Byte)
     chain = Column(SAUnicode, nullable=True)
     tag = Column(SAUnicode, nullable=True)
 
