@@ -26,7 +26,9 @@ from mailman.model.digests import OneLastDigest
 from mailman.model.member import Member
 from mailman.model.preferences import Preferences
 from mailman.model.user import User
+from mailman.utilities.queries import QuerySequence
 from public import public
+from sqlalchemy import or_
 from zope.interface import implementer
 
 
@@ -99,7 +101,7 @@ class UserManager:
     @dbconnection
     def users(self, store):
         """See `IUserManager`."""
-        yield from store.query(User).order_by(User.id).all()
+        return QuerySequence(store.query(User).order_by(User.id))
 
     @dbconnection
     def create_address(self, store, email, display_name=None):
@@ -159,3 +161,13 @@ class UserManager:
         """ See `IUserManager."""
         yield from store.query(User).filter_by(
             is_server_owner=True).order_by(User.id)
+
+    @dbconnection
+    def find_users(self, store, query):
+        """ See `IUserManager."""
+        q = '%{}%'.format(query)
+        yield from store.query(User).join(
+            Address, Address.user_id == User.id).filter(
+                or_(User.display_name.ilike(q),
+                    Address.display_name.ilike(q),
+                    Address.email.ilike(q)))
