@@ -1,4 +1,4 @@
-# Copyright (C) 2016-2020 by the Free Software Foundation, Inc.
+# Copyright (C) 2016-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -245,8 +245,9 @@ def is_reject_or_quarantine(mlist, email, dmarc_domain, org=False):
         if len(dmarcs) > 1:
             elog.error(
                 'RRset of TXT records for %s has %d v=DMARC1 entries; '
-                'testing them all',
+                'ignoring them per RFC 7849',
                 dmarc_domain, len(dmarcs))
+            return False
         for entry in dmarcs:
             mo = re.search(r'\bsp=(\w*)\b', entry, re.IGNORECASE)
             if org and mo:
